// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [c,y] = assert_acoscond ( x )
  // Computes the condition number of the acos function.
  // 
  // Calling Sequence
  //   [c,y] = assert_acoscond ( x )
  // 
  // Parameters
  //   x : a n-by-m matrix of doubles
  //   c : a n-by-m matrix of doubles, the condition number
  //   y : a n-by-m, matrix of doubles, the function values
  // 
  // Description
  // Computes the condition number of the acos function.
  // The acos function has a large condition number if 
  //   <itemizedlist>
  //   <listitem>x is close to 1,</listitem>
  //   <listitem>x is close to -1,</listitem>
  //   </itemizedlist>
  //
  //<latex>
  //\begin{eqnarray}
  //c(x) = \left| \frac{x}{\sqrt{1-x^2}\arccos(x)} \right|
  //\end{eqnarray}
  //</latex>
  // 
  // Examples
  //  [c,y] = assert_acoscond ( 0.5 )        // c~1
  //  [c,y] = assert_acoscond ( 1-%eps/2 )   // c~10^7
  //  [c,y] = assert_acoscond ( -1+%eps/2 )  // c~10^7
  //  assert_acoscond ( -1 ) // %inf
  //  assert_acoscond ( 1 )  // %inf
  //  assert_acoscond ( 0 )  // 0
  //
  //  assert_plotcond ( "acos" , linspace(-1,1,1000) );
  //
  // Authors
  // Michael Baudin, DIGITEO, 2010
  //
  
  [lhs,rhs]=argn()
  if ( rhs<>1 ) then
    errmsg = sprintf("%s: Unexpected number of arguments : %d provided while %d are expected.","assert_acoscond",rhs,1);
    error(errmsg)
  end
  //
  y = acos(x)
  c = zeros(x)
  k = find(abs(x)<>1)
  c(k) = abs(x(k)) ./ abs(sqrt(1-x(k).^2) .* y(k))
  k = find(abs(x)==1)
  c(k) = %inf
endfunction

