// Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [t,msg] = assert_benchfun ( varargin )
  // Benchmarks a function and measure its performance.
  //
  // Calling sequence
  //   [t,msg] = assert_benchfun ( fun )
  //   [t,msg] = assert_benchfun ( fun , iargs )
  //   [t,msg] = assert_benchfun ( fun , iargs , nlhs )
  //   [t,msg] = assert_benchfun ( fun , iargs , nlhs , kmax )
  //   [t,msg] = assert_benchfun ( fun , iargs , nlhs , kmax , name )
  //   [t,msg] = assert_benchfun ( fun , iargs , nlhs , kmax , name , verbose )
  //
  // Parameters
  //   fun : a function, the function to be executed
  //   iargs : a list, the list of input arguments for the function (default iargs=list())
  //   nlhs : a 1 x 1 matrix of floating point integers, the number of output arguments of the function (default nlhs=1)
  //   kmax : a 1 x 1 matrix of floating point integers, the number of executions (default kmax=10)
  //   verbose : a 1x1 matrix of booleans, set verbose = %t to display the message (default verbose=%f)
  //   t : a kmax x 1 matrix of doubles, the system times, in seconds, required to execute the function
  //   msg : a 1 x 1 matrix of strings, a message summarizing the benchmark
  //
  // Description
  //   This function is designed to be used when measuring the 
  //   performance of a function.
  //   It uses the timer function to measure the system time.
  //   The function is executed kmax times and the performance is 
  //   gathered into the matrix t.
  //   The message summarizes the test and contains the mean,
  //   min and max of the times.
  //
  //   Any optional input argument equal to the empty matrix is replaced by its default value.
  //
  // Examples
  // function c = pascalup_col (n)
  //   // Pascal up matrix.
  //   // Column by column version
  //   c = eye(n,n)
  //   c(1,:) = ones(1,n)
  //   for i = 2:(n-1)
  //     c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
  //   end
  // endfunction
  // // Prints the message
  // assert_benchfun ( pascalup_col , list(100) , [] , [] , "pascalup_col" , %t );
  // // Basic call
  // [t,msg] = assert_benchfun ( pascalup_col , list(100) )
  // mprintf("pascalup_col: %s\n",msg);
  // // Customize the number of runs
  // [t,msg] = assert_benchfun ( pascalup_col , list(100) , [] , 15 )
  // // Customize the message
  // [t,msg] = assert_benchfun ( pascalup_col , list(100) , [] , 15 , "pascalup_col" )
  //
  // // Test sin (this is a primitive)
  // assert_benchfun ( sin , list(ones(100,100)) , 1 , 15 , "sin" , %t );
  //
  // // Bench a function with 2 output arguments
  // function [a,b] = testfun ()
  //    a = ones(10,10)
  //    b = zeros(10,10)
  // endfunction
  // [t,msg]=assert_benchfun ( testfun , [] , 2 , 15 , "testfun" , %t );
  //
  // // Bench a function with 2 input arguments
  // function y = testfun2 ( m , n )
  //    y = zeros(m,n)
  // endfunction
  // [t,msg]=assert_benchfun ( testfun2 , list(11,12) , [] , [] , "testfun2" , %t );
  //
  // Authors
  //   2010 - DIGITEO - Michael Baudin
  
  [lhs,rhs] = argn()
  if ( and(rhs<>(1:6)) ) then
    noerrmsg = sprintf(gettext("%s: Unexpected number of arguments : %d provided while %d to %d are expected."),..
      "assert_benchfun",rhs,1,6)
    error(noerrmsg)
  end
  __benchfun_fun__ = varargin(1)
  iargs = argindefault ( rhs , varargin , 2 , list() )
  nlhs = argindefault ( rhs , varargin , 3 , 1 )
  kmax = argindefault ( rhs , varargin , 4 , 10 )
  name = argindefault ( rhs , varargin , 5 , "" )
  verbose = argindefault ( rhs , varargin , 6 , %f )
  //
  // Check type
  if ( and ( typeof(__benchfun_fun__) <> ["function" "list" "fptr"] ) ) then 
    error(sprintf(gettext("%s: Wrong type for input argument #%d: variable %s has type %s while %s is expected.\n"), ..
      "assert_benchfun", 1, "fun" , typeof(__benchfun_fun__) , "function"))
  end
  if ( typeof(iargs) <> "list" ) then 
    error(sprintf(gettext("%s: Wrong type for input argument #%d: variable %s has type %s while %s is expected.\n"), ..
      "assert_benchfun", 2, "nlhs" , typeof(iargs) , "list"))
  end
  if ( typeof(nlhs) <> "constant" ) then 
    error(sprintf(gettext("%s: Wrong type for input argument #%d: variable %s has type %s while %s is expected.\n"), ..
      "assert_benchfun", 3, "nlhs" , typeof(nlhs) , "constant"))
  end
  if ( typeof(kmax) <> "constant" ) then 
    error(sprintf(gettext("%s: Wrong type for input argument #%d: variable %s has type %s while %s is expected.\n"), ..
      "assert_benchfun", 4, "kmax" , typeof(kmax) , "constant"))
  end
  if ( typeof(name) <> "string" ) then 
    error(sprintf(gettext("%s: Wrong type for input argument #%d: variable %s has type %s while %s is expected.\n"), ..
      "assert_benchfun", 5, "name" , typeof(name) , "string"))
  end
  if ( typeof(verbose) <> "boolean" ) then 
    error(sprintf(gettext("%s: Wrong type for input argument #%d: variable %s has type %s while %s is expected.\n"), ..
      "assert_benchfun", 6, "verbose" , typeof(verbose) , "boolean"))
  end
  //
  // Check size
  if ( size(nlhs,"*") <> 1 ) then 
    error(sprintf(gettext("%s: Wrong size for input argument #%d: variable %s has size %d while %d is expected.\n"), ..
      "assert_benchfun", 3, "nlhs" , size(nlhs,"*") , 1))
  end
  if ( size(kmax,"*") <> 1 ) then 
    error(sprintf(gettext("%s: Wrong size for input argument #%d: variable %s has size %d while %d is expected.\n"), ..
      "assert_benchfun", 4, "kmax" , size(kmax,"*") , 1))
  end
  if ( size(name,"*") <> 1 ) then 
    error(sprintf(gettext("%s: Wrong size for input argument #%d: variable %s has size %d while %d is expected.\n"), ..
      "assert_benchfun", 5, "name" , size(name,"*") , 1))
  end
  if ( size(verbose,"*") <> 1 ) then 
    error(sprintf(gettext("%s: Wrong size for input argument #%d: variable %s has size %d while %d is expected.\n"), ..
      "assert_benchfun", 6, "verbose" , size(verbose,"*") , 1))
  end
  //
  // Check content
  if ( nlhs < 1 ) then
    error(sprintf(gettext("%s: Wrong content for input argument #%d: variable %s is expected to be positive, but its content is %s.\n"), ..
      "assert_benchfun", 3, "nlhs" , string(nlhs)))
  end
  if ( kmax < 1 ) then
    error(sprintf(gettext("%s: Wrong content for input argument #%d: variable %s is expected to be positive, but its content is %s.\n"), ..
      "assert_benchfun", 4, "kmax" , kmax))
  end
  //
  // Compute the intstruction string to be launched
  ni = length ( iargs )
  instr = ""
  // Put the LHS arguments
  if ( nlhs > 0 ) then
    instr = instr + "["
  end
  for i = 1 : nlhs
    if ( i > 1 ) then
      instr = instr + ","
    end
    instr = instr + "x" + string(i)
  end
  if ( nlhs > 0 ) then
    instr = instr + "]"
  end
  if ( nlhs > 0 ) then
    instr = instr + "="
  end
  // Put the RHS arguments
  instr = instr + "__benchfun_fun__("
  for i = 1 : ni
    if ( i > 1 ) then
      instr = instr + ","
    end
    instr = instr + "iargs("+string(i)+")"
  end
  instr = instr + ")"
  //
  // Loop over the tests
  for k = 1 : kmax
    // Call the function
    timer()
    ierr = execstr ( instr , "errcatch" )
    t(k) = timer()
    if ( ierr <> 0 ) then
      errmsg = lasterror()
      error(sprintf(gettext("%s: Failed to run function: %s.\n"), ..
        "assert_benchfun", errmsg))
    end
  end
  if ( name <> "" ) then
    name = name + ": "
  end
  msg = msprintf("%s%d iterations, mean=%f, min=%f, max=%f\n",name,kmax,mean(t),min(t),max(t))
  if ( verbose ) then
    mprintf("%s\n",msg)
  end
endfunction
function argin = argindefault ( rhs , vararglist , ivar , default )
  // Returns the value of the input argument #ivar.
  // If this argument was not provided, or was equal to the 
  // empty matrix, returns the default value.
  if ( rhs < ivar ) then
    argin = default
  else
    if ( vararglist(ivar) <> [] ) then
      argin = vararglist(ivar)
    else
      argin = default
    end
  end
endfunction

