// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [ c , y ] = assert_condnum ( varargin )
  // Computes the empirical condition number of the function f at point x.
  //
  // Calling Sequence
  //   [ c , y ] = assert_condnum ( f , x )
  //   [ c , y ] = assert_condnum ( f , x , order )
  //   [ c , y ] = assert_condnum ( f , x , order , h )
  //
  // Parameters
  //   f : a function with header y = f(x)
  //   x : a vector of doubles, the current point
  //   order : a 1x1 matrix of floating point integers, the order (Default order = 2). Available are order= 1, 2, 4, 6. If order==[], then the default order is used.
  //   h : a matrix of doubles, the step. Default tries to be optimal for accuracy. If h==[], then the default h is used.
  //   c : the condition number as the ratio ry/rx
  //   y : the computed value of f at point x
  //
  // Description
  //   Computes the condition number by using an order one finite difference formula.
  //
  //   Any optional input argument equal to the empty matrix is replaced by its default value.
  //
  // Examples
  //   // Show that sin can be ill-conditionned
  //   function [c,y] = sincond (x)
  //     // Condition number of the sin function
  //     y = sin(x)
  //     c = abs(x.*cos(x)) ./ abs(y)
  //   endfunction
  //   [ c , y ] = assert_condnum ( sin , 1.e-100 )         // c~1
  //   [ c , y ] = assert_condnum ( sin , 0 )               // c = nan
  //   [ c , y ] = assert_condnum ( sin , 3.14159 )         // c ~10^6
  //   [ c , y ] = assert_condnum ( sin , 3.141592653 )     // c ~10^9
  //   [ c , y ] = assert_condnum ( sin , 3.1415926535898 ) // c ~10^14
  //   [ c , y ] = assert_condnum ( sin , %pi )             // c ~10^16
  //   [ c , y ] = assert_condnum ( sin , 1.000000357564167061e5 ) // c ~10^16
  //
  //   // An ill-conditionned case for the sum.
  //   function [c,y] = sumcond(x)
  //     // Condition number of the sum function
  //     y = sum(x)
  //     c = sum(abs(x)) / abs(y)
  //   endfunction
  //   xl = 10^(1:15);
  //   x = [-xl xl+0.1];
  //   yExpected = 1.5
  //   cExpected = sumcond(x) // c~10^15
  //   [ c , y ] = assert_condnum ( sum , x )
  //
  //   // Check various orders for the sqrt function.
  //   function [c,y] = sqrtcond(x)
  //     // Condition number of the sqrt function
  //     y = sum(x)
  //     c = 1/2
  //   endfunction
  //   // The condition number is 0.5.
  //   cc = assert_condnum ( sqrt , 1 , 1 )
  //   cc = assert_condnum ( sqrt , 1 , 2 )
  //   cc = assert_condnum ( sqrt , 1 , 4 )
  //   cc = assert_condnum ( sqrt , 1 , 6 )
  //   // Make a loop
  //   x = 1.;
  //   order = [1 2 4 6];
  //   for k = 1:4
  //     cc = assert_condnum ( sqrt , x , order(k) );
  //     ce = sqrtcond ( x );
  //     d = assert_computedigits(cc,ce);
  //     disp([order(k) cc ce d])
  //   end
  //
  // // Use default order, but configure h
  // [cc,yc] = assert_condnum ( sqrt , 1 , [] , 1.e-8 )
  // // Use default order and h
  // [cc,yc] = assert_condnum ( sqrt , 1 , [] , [] )
  // // Set order, use default h
  // [cc,yc] = assert_condnum ( sqrt , 1 , 4 , [] );
  //
  // Authors
  //   Michael Baudin, Digiteo, 2010
  //
  [lhs,rhs]=argn()
  if ( and ( rhs <> [2 3 4] ) ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected number of input arguments : %d provided while %d to %d are expected.") , "assert_condnum" , rhs , 2 , 4 )
    error(errmsg)
  end
  //
  // Get arguments
  __assert_condnum_f = varargin ( 1 )
  x = varargin ( 2 )
  //
  order = argindefault ( rhs , varargin , 3 , 2 )
  h = argindefault ( rhs , varargin , 4 , [] )
  hflag = ( h <> [] )
  //
  // Check types of variables
  if ( and ( typeof(__assert_condnum_f) <> [ "function" "fptr" "list" ] ) ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected type of input argument #%d : variable %s has type %s while %s is expected.") , "assert_condnum" , 1 , "f" , typeof(__assert_condnum_f) , "function" )
    error(errmsg)
  end
  if ( typeof(x) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected type of input argument #%d : variable %s has type %s while %s is expected.") , "assert_condnum" , 2 , "x" , typeof(x) , "constant" )
    error(errmsg)
  end
  if ( typeof(order) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected type of input argument #%d : variable %s has type %s while %s is expected.") , "assert_condnum" , 3 , "order" , typeof(order) , "constant" )
    error(errmsg)
  end
  if ( hflag ) then
  if ( typeof(h) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected type of input argument #%d : variable %s has type %s while %s is expected.") , "assert_condnum" , 4 , "h" , typeof(h) , "constant" )
    error(errmsg)
  end
  end
  //
  // Check size of variables
  if ( size(x,"r") <> 1 & size(x,"c")  <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected size of input argument #%d : variable %s has size [%d,%d] while a vector is expected.") , "assert_condnum" , 2 , "x" , size(x,"r") , size(x,"c") )
    error(errmsg)
  end
  if ( size ( order , "*" )  <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected size of input argument #%d : variable %s has size %d while %d is expected.") , "assert_condnum" , 3 , "order" , size(order,"*") , 1 )
    error(errmsg)
  end
  //
  // Check content of variables
  if ( and ( order <> [1 2 4 6] ) ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected content %s of input argument #%d: %s.") , "assert_condnum" , string(order) , 3 , "order" )
    error(errmsg)
  end
  //
  // Make x a column vector
  x = x(:)
  //
  // Compute the step
  if ( ~hflag ) then
    h = assert_condnum_optimalh ( order ) 
    h = h * (1+1.d-3*abs(x))
    // Another strategy : h = max(h , h*abs(x) )
  end
  y = assert_condnum_evalf ( __assert_condnum_f , x )
  //
  // Check size of y
  if ( size(y,"r") <> 1 & size(y,"c")  <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected size of %s : variable has size %d while %d is expected.") , "assert_condnum" , "y=f(x)" , size(y,"*") , 1 )
    error(errmsg)
  end
  //
  // Compute jacobian
  jac = assert_condnum_firstder ( __assert_condnum_f , x , order , h )
  // Make jac a column vector
  jac = jac(:)
  if ( norm(x) == 0 ) then
    // rx may be %inf or %nan, depending on norm(h)
    rx = norm(h) * %inf 
  else
    rx = norm(h) / norm(x)
  end
  if ( abs(y) == 0 ) then
    // ry may be %inf or %nan, depending on abs(jac' * h)
    ry = abs(jac' * h) * %inf
  else  
    ry = abs(jac' * h) / abs(y)
  end
  if ( rx == 0 ) then
    // c may be %inf or %nan, depending on ry
    c = ry * %inf
  else  
    c = ry / rx
  end
endfunction

function y = assert_condnum_evalf ( __assert_condnum_f , x )
  // Evaluates the function at point x.
  // The function can be a "function" or "fptr" : directly call it.
  // If the function is a list (type=15), the first element is the function, the remaining items are the arguments.
  if ( type(__assert_condnum_f) == 15 ) then
    __assert_condnum_f_f = __assert_condnum_f ( 1 )
    y = __assert_condnum_f_f ( x , __assert_condnum_f(2:$) )
  else
    y = __assert_condnum_f ( x )
  end
endfunction

function jac = assert_condnum_firstder ( __assert_condnum_f , x , order , h )
  // Computes the first derivative (Jacobian matrix) by a finite difference formula
  // http://reference.wolfram.com/mathematica/tutorial/NDSolvePDE.html#c:4 (Reference for order = 6)
  if ( order == 1 ) then
    fx = assert_condnum_evalf ( __assert_condnum_f , x )
    fxp = assert_condnum_evalf ( __assert_condnum_f , x+h )
    jac = (fxp - fx)/h
  elseif ( order == 2 ) then
    fxp = assert_condnum_evalf ( __assert_condnum_f , x+h )
    fxm = assert_condnum_evalf ( __assert_condnum_f , x-h )
    jac = (fxp - fxm)/(2*h)
  elseif ( order == 4 ) then
    fxp = assert_condnum_evalf ( __assert_condnum_f , x+h )
    fxm = assert_condnum_evalf ( __assert_condnum_f , x-h )
    fxpp = assert_condnum_evalf ( __assert_condnum_f , x+2*h )
    fxmm = assert_condnum_evalf ( __assert_condnum_f , x-2*h )
    jac = (-fxpp + 8*fxp - 8*fxm + fxmm)/(12*h)
  elseif ( order == 6 ) then
    fxp = assert_condnum_evalf ( __assert_condnum_f , x+h )
    fxm = assert_condnum_evalf ( __assert_condnum_f , x-h )
    fxpp = assert_condnum_evalf ( __assert_condnum_f , x+2*h )
    fxmm = assert_condnum_evalf ( __assert_condnum_f , x-2*h )
    fxppp = assert_condnum_evalf ( __assert_condnum_f , x+3*h )
    fxmmm = assert_condnum_evalf ( __assert_condnum_f , x-3*h )
    jac = (fxppp - 9*fxpp + 45*fxp - 45*fxm + 9*fxmm - fxmmm)/(60*h)
  else
    errmsg = sprintf ( gettext ( "%s: Unexpected order = %d provided.") , "assert_condnum_firstder" , order )
    error(errmsg)
  end
endfunction
function h = assert_condnum_optimalh ( order )
  // Returns an optimal step h given order
  // alpha1 : the power of h in the bottom term in the finite difference formula : jac = (.*f(.)+.*f(.)+...)/(.*h^alpha1)
  if ( order == 1 ) then
    alpha1 = 1
  elseif ( order == 2 ) then
    alpha1 = 1
  elseif ( order == 4 ) then
    alpha1 = 1
  elseif ( order == 6 ) then
    alpha1 = 1
  else
    errmsg = sprintf ( gettext ( "%s: Unexpected order = %d provided.") , "assert_condnum_optimalh" , order )
    error(errmsg)
  end
  h = %eps^(1/(alpha1+order))
endfunction
function argin = argindefault ( rhs , vararglist , ivar , default )
  // Returns the value of the input argument #ivar.
  // If this argument was not provided, or was equal to the 
  // empty matrix, returns the default value.
  if ( rhs < ivar ) then
    argin = default
  else
    if ( vararglist(ivar) <> [] ) then
      argin = vararglist(ivar)
    else
      argin = default
    end
  end
endfunction

