// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function h = assert_plotcond ( funname , x )
  // Plots the condition number of a function.
  // 
  // Calling Sequence
  //   h = assert_plotcond ( funname )
  // 
  // Parameters
  //   funname : a 1-by-1, matrix of strings, the function name
  //   h : the graphics handle
  // 
  // Description
  // Plots the condition number of the function with the function itself.
  // Automatically switches to log-scale for f and its condition, if possible
  // and if necessary.
  // Move the y-label to the left if x data contains mixed signs.
  // 
  // Examples
  //  // The values of atan and its condition are regular:
  //  // use a regular scale.
  //  assert_plotcond ( "atan" , linspace(-1,1,1000) );
  // 
  //  // The condition values of acos are positive and of very different magnitudes :
  //  // automatically switches to log scale.
  //  assert_plotcond ( "acos" , linspace(-1,1,1000) );
  //  assert_plotcond ( "sin" , linspace(-4,4,1000) );
  //
  //  // The exp values are positive and of very different magnitudes :
  //  // automatically switches to log scale.
  //  assert_plotcond ( "exp" , linspace(-7e2,7.e2,1000) );
  //
  //  // The values of the condition number are very close to zero, 
  //  // but we keep the regular scale, as low condition numbers 
  //  // are of no interest for us.
  //  assert_plotcond ( "erf" , linspace(-6,6,1000) );
  //
  // Authors
  // Michael Baudin, DIGITEO, 2010
  //
  
  [lhs,rhs]=argn()
  if ( rhs<>2 ) then
    errmsg = sprintf("%s: Unexpected number of arguments : %d provided while %d are expected.","assert_plotcond",rhs,2);
    error(errmsg)
  end
  //
  h = scf()
  subplot(2,1,1)
  // Plot f
  instr = "plot(x,"+funname+")"
  ierr = execstr(instr,"errcatch")
  if ( ierr <> 0 ) then
    le = lasterror()
    errmsg = sprintf("%s: %s","assert_plotcond",le);
    error(errmsg)
  end
  // Set title
  instr = "xtitle(""" + funname + """,""x"",""" + funname + "(x)"")"
  ierr = execstr(instr,"errcatch")
  if ( ierr <> 0 ) then
    le = lasterror()
    errmsg = sprintf("%s: %s","assert_plotcond",le);
    error(errmsg)
  end
  subplot(2,1,2)
  // Plot condition
  instr = "plot(x,assert_" + funname + "cond)"
  ierr = execstr(instr,"errcatch")
  if ( ierr <> 0 ) then
    le = lasterror()
    errmsg = sprintf("%s: %s","assert_plotcond",le);
    error(errmsg)
  end
  // Set title
  instr = "xtitle(""Conditionning of " + funname + """,""x"",""Condition Number"")"
  ierr = execstr(instr,"errcatch")
  if ( ierr <> 0 ) then
    le = lasterror()
    errmsg = sprintf("%s: %s","assert_plotcond",le);
    error(errmsg)
  end
  // Set log scale for the function, if possible and if necessary
  f = h.children(2).children.children.data(:,2);
  f(find(f==0)) = [];
  if ( find(f<0) == [] ) then
    fmax = max(round(log10(f)))
    fmin = min(round(log10(f)))
    if ( fmax > fmin + 2 ) then
      h.children(2).log_flags = "nln"
    end
  end
  // Set log scale for the condition, if necessary
  c = h.children(1).children.children.data(:,2)
  c(find(c==0)) = []
  cmax = max(round(log10(c)))
  cmin = min(round(log10(c)))
  if ( cmax > 1.e1 & cmax > cmin + 2 ) then
    h.children(1).log_flags = "nln"
  end
  // Push the y-label to the left, if necessary
  if ( min(x)*max(x) < 0 ) then
    h.children(1).y_location="left"
    h.children(2).y_location="left"
  end
endfunction

