// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function strmat = assert_printtable ( varargin )
  //   Prints a table of data with a given number of digits.
  //
  // Calling Sequence
  //   strmat = assert_printtable ( table )
  //   strmat = assert_printtable ( table , digits )
  //   strmat = assert_printtable ( table , digits , prntbool )
  //   strmat = assert_printtable ( table , digits , prntbool , sep )
  // 
  // Parameters
  //   table : a mxn matrix of doubles, the table of values
  //   digits : a 1x1 matrix of floating point integers, the number of digits to display (default digits=17)
  //   prntbool : a 1x1 matrix of booleans, prntbool=%t to print the table in the console (default prntbool=%f)
  //   sep : a 1x1 matrix of strings, the separator between the columns (default sep=" ")
  //   strmat : a nx1 matrix of strings, the formatted table
  //
  // Description
  //   Prints a table of data with a given number of digits. 
  //
  //   Any optional input argument equal to the empty matrix is replaced by its default value.
  //
  //   Notice that, if the exponent requires 2 digits or less, then the number of displayed 
  //   digits after the decimal point is digits+1. 
  //   If the exponent requires 3 digits, then the number of displayed digits 
  //   after the decimal point is exactly digits.
  //   This is a consequence of the inconsistency of the format function.
  //
  //   The goal of this function is to provide a portable way of creating tables of 
  //   data, for creation of unit tests.
  //
  //   We do not use the msprintf function, because this function produces a different 
  //   number of digits in the exponent on Windows and on Linux.
  //
  //   This function allows to display values in a more numerically consistent way 
  //   than the built-in format and disp functions.
  //   For example, the number of digits in the format function do not correspond 
  //   to the number of digits after the decimal point.
  //   The printtable function manages this transparently for the user.
  //
  //   The current function also manages the printing more consistently than 
  //   the sci2exp function, which does not manage the number of digits.
  //
  //   Special IEEE values such as %nan and %inf are displayed as "%inf" and "%nan", so that the 
  //   output of the printtable function can be directly inserted into a script.
  //   This is different from the output of the disp function, where %inf entries are 
  //   displayed as "Infinity" and %nan entries are displayed as "NaN", making the 
  //   copy and paste difficult.
  //
  //   Moreover, the formatting of the entries in the matrix is done entry-by-entry, 
  //   so that values of varying exponents can be managed easily (see the 3.15e300 value in the 
  //   example below).
  //
  // Examples
  // table = [
  //  1 0 200 %inf 0
  //  1 1.e-300 200 %inf 0
  //  9.99999999999990010e-001 9.99999999999999980e-201 200 3.15e300 102
  //  9.99999999899999990e-001 1.e-100 200 296 117
  //  1 %inf -%inf %nan 0
  // ];
  // strmat = assert_printtable ( table )
  // strmat = assert_printtable ( table , 17 )
  // strmat = assert_printtable ( table , 17 , %t )
  // strmat = assert_printtable ( table , 17 , %t , "," )
  // strmat = assert_printtable ( table , 15 )
  // strmat = assert_printtable ( table , 8 )
  //
  // Authors
  // Michael Baudin, DIGITEO, 2010

  [lhs,rhs]=argn()
  if ( and ( rhs <> [ 1 2 3 4 ] ) ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected number of input arguments : %d provided while %d to %d are expected.") , "assert_printtable" , rhs , 1 , 4 )
    error(errmsg)
  end
  //
  // Read arguments
  table = varargin(1)
  digits = argindefault ( rhs , varargin , 2 , 17 )
  prntbool = argindefault ( rhs , varargin , 3 , %f )
  sep = argindefault ( rhs , varargin , 4 , " " )
  //
  // Check types of variables
  if ( typeof(table) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Expected type ""%s"" for input argument %s #%d, but got %s instead.") , "assert_printtable" , "constant" , "table" , 1 , typeof(table) )
    error(errmsg)
  end
  if ( typeof(digits) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Expected type ""%s"" for input argument %s #%d, but got %s instead.") , "assert_printtable" , "constant" , "digits" , 2 , typeof(digits) )
    error(errmsg)
  end
  if ( typeof(prntbool) <> "boolean" ) then
    errmsg = sprintf ( gettext ( "%s: Expected type ""%s"" for input argument %s #%d, but got %s instead.") , "assert_printtable" , "boolean" , "prntbool" , 3 , typeof(prntbool) )
    error(errmsg)
  end
  if ( typeof(sep) <> "string" ) then
    errmsg = sprintf ( gettext ( "%s: Expected type ""%s"" for input argument %s #%d, but got %s instead.") , "assert_printtable" , "string" , "sep" , 4 , typeof(sep) )
    error(errmsg)
  end
  //
  // Check sizes of variables
  if ( size(digits,"*") <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Wrong size for input argument #%d: %s: Expected size %d, but got %d instead.") , "assert_printtable" , 2 , "digits" , 1 , size(digits,"*") )
    error(errmsg)
  end
  if ( size(prntbool,"*") <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Wrong size for input argument #%d: %s: Expected size %d, but got %d instead.") , "assert_printtable" , 3 , "prntbool" , 1 , size(prntbool,"*") )
    error(errmsg)
  end
  if ( size(sep,"*") <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Wrong size for input argument #%d: %s: Expected size %d, but got %d instead.") , "assert_printtable" , 4 , "sep" , 1 , size(sep,"*") )
    error(errmsg)
  end
  //
  // Check content of variables
  if ( digits<1 | digits>17 ) then
    errmsg = sprintf ( gettext ( "%s: Wrong value for input argument #%d: %s: Expected from %d to %d, but got %d instead.") , "assert_printtable" , 2 , "digits" , 1 , 17 , digits )
    error(errmsg)
  end
  if ( round(digits)<>digits ) then
    errmsg = sprintf ( gettext ( "%s: Wrong value for input argument #%d: %s: Expected floating point integer, but got %f instead.") , "assert_printtable" , 2 , "digits" , digits )
    error(errmsg)
  end
  //
  // Proceed...
  // Backup the format
  fmtback = format()
  format("e",digits+8)
  nrows = size(table,"r")
  ncols = size(table,"c")
  formatS1Neg = "%" + msprintf("%d",digits+7) + "s"
  formatS2Neg = sep + formatS1Neg
  formatS1Pos = " %" + msprintf("%d",digits+7) + "s"
  formatS2Pos = sep + formatS1Pos
  strmat = []
  for i = 1 : nrows
    str = ""
    for j = 1 : ncols
      if ( table(i,j) == %inf | table(i,j) == -%inf | isnan(table(i,j)) ) then
        if ( j == 1 ) then
          formatS = formatS1Pos
        else
          formatS = formatS2Pos
        end
      else
        if ( table(i,j) >= 0 ) then
          if ( j == 1 ) then
            formatS = formatS1Pos
          else
            formatS = formatS2Pos
          end
        else
          if ( j == 1 ) then
            formatS = formatS1Neg
          else
            formatS = formatS2Neg
          end
        end
      end
      if ( table(i,j) == %inf ) then
        str = str + msprintf(formatS,"%inf")
      elseif ( table(i,j) == -%inf ) then
        str = str + msprintf(formatS,"-%inf")
      elseif ( isnan(table(i,j)) ) then
        str = str + msprintf(formatS,"%nan")
      else
        str = str + msprintf(formatS,string(table(i,j)))
      end
    end
    strmat(i) = str
    if ( prntbool ) then
      mprintf("%s\n",str)
    end
  end
  // Restore the format
  if ( fmtback(1)==0 ) then
    format("e",fmtback(2))
  else
    format("v",fmtback(2))
  end
endfunction
function argin = argindefault ( rhs , vararglist , ivar , default )
  // Returns the value of the input argument #ivar.
  // If this argument was not provided, or was equal to the 
  // empty matrix, returns the default value.
  if ( rhs < ivar ) then
    argin = default
  else
    if ( vararglist(ivar) <> [] ) then
      argin = vararglist(ivar)
    else
      argin = default
    end
  end
endfunction

