// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [c,y] = assert_tancond ( x )
  // Computes the condition number of the tan function.
  // 
  // Calling Sequence
  //   [c,y] = assert_tancond ( x )
  // 
  // Parameters
  //   x : a n-by-m matrix of doubles
  //   c : a n-by-m matrix of doubles, the condition number
  //   y : a n-by-m, matrix of doubles, the function values
  // 
  // Description
  // Computes the condition number of the tan function.
  // The tan function has a large condition number if 
  //   <itemizedlist>
  //   <listitem>x is large,</listitem>
  //   <listitem>x is an integer multiple of pi (where sin(x)=0),</listitem>
  //   <listitem>x-pi/2 is an integer multiple of pi (where cos(x)=0).</listitem>
  //   </itemizedlist>
  //
  //<latex>
  //\begin{eqnarray}
  //c(x) = \left| \frac{x}{\cos(x) \sin(x)} \right|
  //\end{eqnarray}
  //</latex>
  // 
  // Examples
  //  [c,y] = assert_tancond ( 1 )
  //  [c,y] = assert_tancond ( 0 )     // 1
  //  [c,y] = assert_tancond ( %pi/2 )
  //  [c,y] = assert_tancond ( %pi )
  //
  //  assert_plotcond ( "tan" , linspace(-4,4,1000) );
  //
  // Authors
  // Michael Baudin, DIGITEO, 2010
  //
  
  [lhs,rhs]=argn()
  if ( rhs<>1 ) then
    errmsg = sprintf("%s: Unexpected number of arguments : %d provided while %d are expected.","assert_tancond",rhs,1);
    error(errmsg)
  end
  //
  y = tan(x)
  c = zeros(x)
  k = find(x<>0)
  c(k) = abs(x(k)) ./ abs(cos(x(k)).^2.*y(k))
  k = find(x==0)
  c(k) = 1
endfunction

