// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

function flag = MY_assert_equal ( computed , expected )
  if (and(isnan([computed expected]))) then
    flag = 1
    return
  elseif computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = MY_assert_close ( computed, expected, epsilon )
  if (expected==0.0) then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

function [c,y] = sincond (x)
  // Condition number of the sin function
  y = sin(x)
  c = zeros(x)
  k = find(x<>0)
  c(k) = abs(x(k).*cos(x(k))) ./ abs(y(k))
  k = find(x==0)
  c(k) = 1
endfunction

function [c,y] = sumcond(x)
  // Condition number of the sum function
  y = sum(x)
  if ( abs(y) == 0 ) then
    c = %inf
  else
    c = sum(abs(x)) / abs(y)
  end
endfunction

function [c,y] = sqrtcond(x)
  // Condition number of the sqrt function
  y = sqrt(x)
  c = 1/2
endfunction

x = 1.e-100;
[cc,yc] = assert_condnum ( sin , x );
[ce,ye] = sincond ( x );
MY_assert_close ( cc , ce , 1.e-11 );
MY_assert_equal ( yc , ye );
//
x = 0;
[cc,yc] = assert_condnum ( sin , x );
//[ce,ye] = sincond ( x );
// The condnum function cannot remove the singularity as sincond does.
MY_assert_equal ( isnan(cc) , %t ); 
MY_assert_equal ( yc , 0 );
//
x = 3.14159
[cc,yc] = assert_condnum ( sin , x );
[ce,ye] = sincond ( x );
MY_assert_close ( cc , ce , 1.e-10 );
MY_assert_equal ( yc , ye );
//
x = 3.141592653;
[cc,yc] = assert_condnum ( sin , x );
[ce,ye] = sincond ( x );
MY_assert_close ( cc , ce , 1.e-10 );
MY_assert_equal ( yc , ye );
//
x = 3.1415926535898;
[cc,yc] = assert_condnum ( sin , x );
[ce,ye] = sincond ( x );
MY_assert_close ( cc , ce , 1.e-10 );
MY_assert_equal ( yc , ye );
//
x = %pi;
[cc,yc] = assert_condnum ( sin , x );
[ce,ye] = sincond ( x );
MY_assert_close ( cc , ce , 1.e-10 );
MY_assert_equal ( yc , ye );
//
x = 1.000000357564167061e5;
[cc,yc] = assert_condnum ( sin , x );
[ce,ye] = sincond ( x );
MY_assert_close ( cc , ce , 1.e-6 );
MY_assert_equal ( yc , ye );
//

//
// An ill-conditionned case for the sum.
xl = 10^(1:15);
x = [-xl xl+0.1];
[cc,yc] = assert_condnum ( sum , x );
//[ce,ye] = sumcond(x); // ce = 1.269841270D+15
MY_assert_equal ( floor(log10(cc)) , 15 );
MY_assert_close ( yc , 1.5 , 1.e-0 );
//
x = 1.;
[cc,yc] = assert_condnum ( sqrt , x );
[ce,ye] = sqrtcond ( x );
MY_assert_close ( cc , ce , 1.e-11 );
MY_assert_equal ( yc , ye );
//
// Check various orders
x = 1.;
precision = [1.e-7 1.e-11 1.e-12 1.e-13];
order = [1 2 4 6];
[ce,ye] = sqrtcond ( x );
for k = 1:4
  [cc,yc] = assert_condnum ( sqrt , x , order(k) );
  MY_assert_close ( cc , ce , precision(k) );
  MY_assert_equal ( yc , ye );
end
//
// Use default order, but configure h
[cc,yc] = assert_condnum ( sqrt , 1 , [] , 1.e-8 );
[ce,ye] = sqrtcond ( x );
MY_assert_close ( cc , ce , 1.e-8 );
MY_assert_equal ( yc , ye );
//
// Use default order and h
[cc,yc] = assert_condnum ( sqrt , 1 , [] , [] );
[ce,ye] = sqrtcond ( x );
MY_assert_close ( cc , ce , 1.e-11 );
MY_assert_equal ( yc , ye );
//
// Set order, use default h
[cc,yc] = assert_condnum ( sqrt , 1 , 4 , [] );
[ce,ye] = sqrtcond ( x );
MY_assert_close ( cc , ce , 1.e-12 );
MY_assert_equal ( yc , ye );
//
// Check various steps
x = 1.;
steps     = [1.e-4 1.e-6  1.e-8 1.e-10];
precision = [1.e-8 1.e-10 1.e-8 1.e-7];
[ce,ye] = sqrtcond ( x );
for k = 1:4
  [cc,yc] = assert_condnum ( sqrt , x , [] , steps(k) );
  MY_assert_close ( cc , ce , precision(k) );
  MY_assert_equal ( yc , ye );
end
//
function y = myfun ( x , a , b , c )
  y = sin(a*x^2+b*x+c)
endfunction
function [c,y] = myfuncond(x,a,b,c)
  // Condition number of the myfun function
  y = myfun ( x , a , b , c )
  fp = cos(a*x^2+b*x+c) * (2*a*x+b)
  c = abs(x*fp)/abs(y)
endfunction
x = 1;
f = list(myfun,1,2,3);
order = [1 2 4 6];
ye = myfun(x,1,2,3);
[ce,ye] = myfuncond(x,1,2,3);
precision = [1.e-7 1.e-10 1.e-12 1.e-11];
for k = 1 : 4
  [cc,yc] = assert_condnum ( f , x , order(k) );
  MY_assert_close ( cc , ce , precision(k) );
  MY_assert_equal ( yc , ye );
end

